@echo off & setlocal enableextensions enabledelayedexpansion
::
:: editArticleHtml.bat
::
:: This file is part of the AccuWebPublish project to publish journal articles on accu.org.
::
:: Contributors: Martin Moene
::
:: $Id: $
::

:: [ToDo]
:: enable usage: editArticleHtml CVu181\*.html
:: without including generated files as input

::
:: Usage:
:: prompt>editArticleHtml [options] file...
::
:: Example
:: prompt>editArticleHtml CVu181\CVu181-Glassborow-Scribbles.html
::
:: Issue "editArticleHtml --help" for more information on usage.
::
:: editArticleHtml uses
:: - html tidy (25 March 2009), http://tidy.sourceforge.net/,
::   Windows excutable, http://www.paehl.com/open_source/?HTML_Tidy_for_Windows
:: - Python 2.6 (not yet 3.0), http://www.python.org/
::
:: For examples of batch programming, see:
::    http://www.netikka.net/tsneti/info/tscmd.php
::

::
:: A. Configuration
::
set          version=1.0
set      versionDate=20 May 2010

set         logLevel=info
set    headingOffset=-2

set       scriptFile=editArticleHtml.py
set htmlTemplateFile=article-template.html

set         python26=C:\Programs\Python26\python.exe
set         python30=C:\Programs\Python30\python.exe
set           python=python

set        scriptDir=%~dp0
set       scriptPath=%scriptDir%%scriptFile%
set htmlTemplatePath=%scriptDir%%htmlTemplateFile%

set      htmlTidyCmd=call %scriptDir%tidy.exe
set       editCVuCmd=call %python% %scriptPath%

::
:: B. Process Options
::
:: --help, -h
:: --version
:: --quiet, -q
:: --verbose, -v
:: --adjust-headings={offset}
:: --html-template={file}
:: --log-level={level}
::
:: See Config section above for default values.
::
set programName=%0

set /a optCount=0
:OptLoop
set argv=%1
set dash=%argv:~0,1%
if     [%argv%]==[]  goto :EndOptLoop
if not [%dash%]==[-] goto :EndOptLoop
   set /a optCount+=1

   if %argv%==-h (
      set optHelp=true
   ) else (

   if %argv%==--help (
      set optHelp=true
   ) else (

   if %argv%==--version (
      set optVersion=true
   ) else (

   if %argv%==-q (
      set optQuiet=true
   ) else (

   if %argv%==--quiet (
      set optQuiet=true
   ) else (

   if %argv%==-v (
      set optVerbose=true
   ) else (

   if %argv%==--verbose (
      set optVerbose=true
   ) else (

   if %argv%==--adjust-headings (
      set optHeadingOffset=true
      set headingOffset=%2
      echo. %2 |findstr /r "[0-9]" >nul
      if errorlevel 1 (
         call :Message "numeric argument expected for option %argv%, got '%2'; see --help for usage."
         goto :ErrorExit
      )
      shift
   ) else (

   if %argv%==--edit (
      set optEdit=true
      set editFilePath=%2
      if not [%~x2]==[.txt] (
         call :Message "text file argument expected for option %argv%, got '%2'; see --help for usage."
         goto :ErrorExit
      )
      shift
   ) else (

   if %argv%==--html-template (
      set optHtmlTemplate=true
      set htmlTemplatePath=%2
      if not [%~x2]==[.html] (
         call :Message "html file argument expected for option %argv%, got '%2'; see --help for usage."
         goto :ErrorExit
      )
      shift
   ) else (

   if %argv%==--log-level (
      set optLogLevel=true
      set logLevel=%2
      echo. "debug,info,warning,error,critical" |findstr /l "%2" >nul
      if errorlevel 1 (
         call :Message "invalid argument for option %argv%, got '%2'; see --help for usage."
         goto :ErrorExit
      )
      shift
   ) else (
      call :Message "unrecognised option '%argv%'; see --help for usage."
      goto :ErrorExit
   ) ) ) ) ) ) ) ) ) ) )
   shift
   goto :OptLoop
:EndOptLoop

::
:: Summarise, perform actions for collected options
::
:Summarise
if defined optHelp (
   call :Usage %programName% & exit/b 0
)
if defined optVersion (
   call :Version %programName% & call %editCVuCmd% --version & %htmlTidyCmd% -v & exit/b 0
)
if [%logLevel%]==[debug] (
   call :Message "optHeadingOffset: '%optHeadingOffset%', headingOffset: '%headingOffset%'"
   call :Message "optHeadingOffset: '%optHtmlTemplate%', headingOffset: '%htmlTemplate%'"
   call :Message "optLogLevel: '%optLogLevel%', logLevel: '%logLevel%'"
   call :Message "optQuiet: '%optQuiet%'"
   call :Message "optVerbose: '%optVerbose%'"
)

::
:: C. Process Files
:: Note: can use wildcards
::
:ProcessFileArgs
if not defined optQuiet if [%1]==[] (
   call :Message "no files specified to process."
   goto :EndProcessFileArgs
)

set /a fileCount=0
:ArgLoop
   if [%1]==[] goto :EndArgLoop
   for %%f in (%1) do (
      if exist %1  (
         set /a fileCount+=1
         call :Process %%f
      ) else (
         call :Message "cannot open file '%%f'."
         goto :ErrorExit
      )
   )
   shift
   goto :ArgLoop
:EndArgLoop

if not defined optQuiet (
   if [%fileCount%]==[0] (
      call :Message "no matching files found."
   ) else (
      call :Message "%fileCount% file(s) processed."
   )
)

:EndProcessFileArgs
endlocal & goto :EOF

::-------------------------------------
:: D. Subroutines
::

::
:: Subroutine Process( file )
:: Process a single HTML file:
::
:Process file
setlocal

set         inputPath=%1
set         inputRoot=%~dp1
set         inputBase=%~n1
set         outputDir=%inputRoot%Output
set        outputBase=%outputDir%\%inputBase%
set     outputLogPath=%outputBase%.log
set    outputTidyPath=%outputBase%-tidy%~x1
set  outputScriptPath=%outputBase%-tidy-edited%~x1
set  outputPrettyPath=%outputBase%-tidy-edited-pretty%~x1

call :Basename "%inputPath%" inputFile

if [%loglevel%]==[debug] (
   :: report variable contents:
   echo Vars:
   echo  scriptDir: %scriptDir%
   echo  scriptPath: %scriptPath%
   echo  htmlTemplatePath: %htmlTemplatePath%
   echo  logLevel: %logLevel%
   echo  inputPath: %inputPath%
   echo  inputRoot: %inputRoot%
   echo  inputBase: %inputBase%
   echo  outputDir: %outputDir%
   echo  outputBase: %outputBase%
   echo  outputLogPath: %outputLogPath%
   echo  outputTidyPath: %outputTidyPath%
   echo  outputScriptPath: %outputScriptPath%
   echo  outputPrettyPath: %outputPrettyPath%
   echo.
   echo  htmlTidyCmd: %htmlTidyCmd%
   echo  editCVuCmd: %editCVuCmd%
)

::
:: 0: Start processing:
::
if not defined optQuiet (
   if defined optVerbose (
      call :Message "editing %inputPath% to %outputPrettyPath%:"
      call :Message "editing %inputPath% to %outputPrettyPath%:" >%outputLogPath%
   ) else (
      echo %inputFile%:
   )
)

if not exist %outputDir% (
   mkdir %outputDir%
   if errorlevel 1 (
      call :Message "cannot create output directory '%outputDir%'."
      goto :ErrorExit
   )
)

::
:: 1. Let html tidy:
:: - correct some errors for us, especially <b>, </b> inbalance,
:: - drop the font and center tags
:: - show only the body part
::
echo %htmlTidyCmd%  --drop-font-tags yes  --show-body-only yes  -output "%outputTidyPath%"  "%inputPath%"   >>"%outputLogPath%"
     %htmlTidyCmd%  --drop-font-tags yes  --show-body-only yes  -output "%outputTidyPath%"  "%inputPath%"  2>>"%outputLogPath%"
::
:: exit value
:: - 2: there where errors
:: - 1: there where warnings
:: - 0: the input file was processed witout warnings or errors.
if errorlevel 2 (
   call :Message "error(s) reported by html tidy (stripping, correcting)."
   goto :ErrorExit
)
if defined optVerbose if errorlevel 1 (
   call :Message "warning(s) reported by html tidy (stripping, correcting)."
)

::
:: 2. let editArticleHtml:
:: - remove all property-value pairs from most html tags, keep only for some tags (cleanHtmlTags)
:: - remove certain html tags, such as div and font (removeHtmlTags)
:: - adjust header levels, e.g. from h4 to h2 etc. (adjustHeadings)
:: - add property-value pairs to certain html tags (decorateHtmlTags)
:: - (not yet: removeFooter)
:: - create the ACCU website middle column with corresponding style files
::
if not defined optVerbose set optQuietPy=--quiet
if defined optEdit set optEditPy=--edit=%editFilePath%

echo %editCVuCmd%  %optQuietPy%  --logging=%logLevel%  --adjust-headings=%headingOffset%  --template="%htmlTemplatePath%"  "%optEditPy%"  "%outputTidyPath%"   >>"%outputLogPath%"
     %editCVuCmd%  %optQuietPy%  --logging=%logLevel%  --adjust-headings=%headingOffset%  --template="%htmlTemplatePath%"  "%optEditPy%"  "%outputTidyPath%"  2>>"%outputLogPath%"
::
:: exit value
:: - 2: there where errors
:: - 1: there where warnings
:: - 0: the input file was processed witout warnings or errors.
if errorlevel 2 (
   call :Message "error(s) reported by %scriptFile%."
   goto :ErrorExit
)
if defined optVerbose if errorlevel 1 (
   call :Message "warning(s) reported by %scriptFile%."
)

::
:: 3. let html tidy pretty format the result:
:: - don't wrap lines
:: - insert empty line between paragraphs
::
echo %htmlTidyCmd%  --output-html yes --wrap 0  --vertical-space yes  "%outputScriptPath%"  > "%outputPrettyPath%"   >>"%outputLogPath%"
     %htmlTidyCmd%  --output-html yes --wrap 0  --vertical-space yes  "%outputScriptPath%"  > "%outputPrettyPath%"  2>>"%outputLogPath%"
::
:: exit value
:: - 2: there where errors
:: - 1: there where warnings
:: - 0: the input file was processed witout warnings or errors.
if errorlevel 2 (
   call :Message "error(s) reported by html tidy (pretty printing)."
   goto :ErrorExit
)
if defined optVerbose if errorlevel 1 (
   call :Message "warning(s) reported by html tidy (pretty printing)."
)

endlocal & goto :EOF
::
:: End of Subroutine Process( file )
::

::
:: Subroutine Version( program_name )
:: Show program's version.
::
:Version program_name
setlocal
echo.
echo %1  %version% (%versionDate%)  Edit ACCU journal html for publication on accu.org
endlocal & goto :EOF

::
:: Subroutine Usage( program_name )
:: Show program's usage.
::
:Usage program_name
setlocal
echo.
echo Usage: %1 [options] file...
echo.
echo Options
echo.
echo --help, -h
echo        this help page
echo.
echo --version
echo        report the versions of this script, python script and html tidy.
echo.
echo --quiet, -q
echo        suppress reporting filenames and number of files processed.
echo.
echo --verbose, -v
echo        also report paths of intermediate files and number of files processed.
echo.
echo --adjust-headings={offset}
echo        change the heading level by the given offset [+0].
echo.
echo --edit={file}
echo        use given file with re.sub() regular exptressions to edit the
echo        article text [].
echo.
echo --html-template={file}
echo        use given file as html template; the string @Article_Body@ in
echo        the template indicates the position where the content will be
echo        placed [scriptdir\article-template.html].
echo.
echo --log-level={level}
echo        specify as one of debug, info, warning, error, critical [info].
echo.
echo EditCVuHtml is a partly-automated process to convert ACCU journal articles in
echo PDF format to HTML for use on the ACCU website (http://accu.org/).
echo.
echo This script converts HTML 3.2 generated with Adobe Acrobat 8 into clean
echo HTML so that it displays as the middle column with an article on the
echo ACCU Website.
echo.
echo The body part of this HTML can then be used as input to the 'body' input
echo field of the New/Edit Journal Articles form on the ACCU website.
echo.
echo How to obtain HTML from PDF:
echo - Load ACCU journal issue PDF in Adobe Acrobat Professional (8)
echo.
echo - Extract relevant pages to new PDF, CVu{Vol}{Issue}-{Author}-{Title},
echo   menu: Document ^> Extract Pages...
echo.
echo - Load just created article PDF
echo.
echo - Export article PDF to HTML 3.2,
echo   menu File ^> Export ^> HTML ^> HTML 3.2
echo   note: unfortunately code sections appear as paragraphs
echo.
echo - Edit raw HTML with batch program editArticleHtml.bat, for example
echo   prompt^>EditCVuHtml\editArticleHtml.bat CVu216\CVu216-Wood-Fsharp.html
echo   this yields CVu216\CVu216-Wood-Fsharp-tidy-edited-pretty.html that
echo   looks like an article's center column on the ACCU webage
echo.
echo - hand-edit for the final result:
echo   - remove text and images that do not belong to the article
echo   - add links and anchors for references
echo   - change code paragraphs into ^<pre class="programlisting"^>
echo   - change code listings into tables with ^<pre class="programlisting"^>
echo   - etc.
endlocal & goto :EOF

::
:: Subroutine Basename( path, basename )
::
:Basename
setlocal enableextensions
for %%f in (%1) do set basename=%%~nxf
endlocal & set %2=%basename% & goto :EOF

::
:: Subroutine Message( message )
:: strip quotes from message
::
:Message
setlocal
set message=%1
set message=%message:"=%
echo %programName%: %message%
endlocal & goto :EOF

:ErrorExit
endlocal
exit /b 1

::
:: end of file
::
